@@ -30,8 +30,7 @@
 #include <hildon-im-ui.h>
 
 #define G_DEBUG_DISABLE 1
-#define BUTTON_WIDTH 80
-#define BUTTON_HIGHT 60
+#define BUTTON_HEIGHT 60
 #define HILDON_IM_FCITX_FKB_TYPE hildon_im_fcitx_fkb_get_type()
 #define HILDON_IM_FCITX_FKB(obj) GTK_CHECK_CAST(obj, hildon_im_fcitx_fkb_get_type(), HildonIMFcitxFKB)
 #define HILDON_IM_FCITX_FKB_CLASS(klass) \
@@ -112,7 +111,7 @@ typedef struct
   gint timeout_handler;
   gint deleted;
 
-  gboolean return_close;
+  HildonGtkInputMode input_mode;
 
   GtkWidget *vbox;
 
@@ -437,60 +436,6 @@ hildon_im_fcitx_fkb_set_property (GObject *object,
       break;
   }
 }
-static gboolean
-get_x11win_prop( Window win){
-    Display* dpy;
-    Atom* prop_list;
-    int prop_count = 0;
-    Window root;
-    Window parent;
-    Window* child_list;
-    unsigned int child_count;
-
-//    g_debug("x start ");
-    dpy = GDK_DISPLAY();
-//    g_debug("check_x11win_classhint start win id = %d", win);
-    dpy = GDK_DISPLAY();
-    XQueryTree( dpy, win, &root, &parent, &child_list, &child_count );
-    XFree(child_list);
-//    g_debug("check_x11win_classhint query root = %d, parent = %d", root, parent );
-    prop_list = XListProperties( dpy, win, &prop_count );
-//    g_debug("check_x11win_classhint get prop_count =  %d", prop_count );
-
-    if ( ( root == parent ) && ( prop_count <= 0 ) ) {
-//        g_debug("check_x11win_classhint find bt mircoB");
-        XFree(prop_list);
-        return FALSE;
-    }
-
-    while( prop_count == 0 && root != parent ){
-        XFree(prop_list);
-        win = parent;
-        XQueryTree( dpy, win, &root, &parent, &child_list, &child_count );
-        XFree(child_list);
-//        g_debug("check_x11win_classhint query root = %d, parent = %d", root, parent );
-        prop_list = XListProperties( dpy, win, &prop_count );
-//        g_debug("check_x11win_classhint get prop_count =  %d", prop_count );
-    }
-    XFree(prop_list);
-    XClassHint classhint;
-    XGetClassHint( dpy, win, &classhint );
-    if ( classhint.res_name == NULL && classhint.res_class == NULL ){
-//        g_debug( "check_x11win_classhint get undefined win" );
-    }
-    else
-        g_debug( "check_x11win_classhint res_name = %s, res_class = %s", classhint.res_name, classhint.res_class );
-    if ( g_strcmp0( classhint.res_class, "Browser" ) == 0 || g_strcmp0( classhint.res_class, "Osso-xterm" ) == 0){
-//        g_debug("check_x11win_classhint find browser or xterminal");
-        return TRUE;
-    }
-    XFree( classhint.res_name );
-    XFree( classhint.res_class );
-
-//    g_debug("check_x11win_classhint end");
-    return FALSE;
-}
-
 #ifndef G_DEBUG_DISABLE
 static void LogToFile (const gchar *log_domain, GLogLevelFlags log_level, const gchar *message, gpointer user_data)
 {
@@ -525,18 +470,25 @@ hildon_im_fcitx_fkb_enable (HildonIMPlugin *plugin,gboolean init)
 #ifndef G_DEBUG_DISABLE
   g_log_set_handler (NULL, G_LOG_LEVEL_DEBUG, LogToFile, NULL);
 #endif
+  if (!window_is_new && !GTK_WIDGET_VISIBLE(GTK_WIDGET(priv->window)))
+  {
+    gtk_widget_destroy(priv->window);
+    window_is_new = TRUE;
+  }
+  hildon_im_ui_play_sound(priv->ui, HILDON_IM_FINGER_TRIGGER_SOUND);
   if (window_is_new)
   {
     priv->window = gtk_window_new (GTK_WINDOW_TOPLEVEL);
-    gtk_window_set_type_hint(GTK_WINDOW(priv->window), GDK_WINDOW_TYPE_HINT_DIALOG);
-    /*gtk_window_set_default_size(GTK_WINDOW(priv->window),800,480);*/
-    gtk_window_set_decorated (GTK_WINDOW (priv->window), FALSE);
+    gtk_window_set_type_hint(GTK_WINDOW(priv->window), GDK_WINDOW_TYPE_HINT_DIALOG | GDK_WINDOW_TYPE_HINT_NOTIFICATION);
     gtk_window_fullscreen(GTK_WINDOW (priv->window));
-    /*g_signal_connect(priv->window, "destroy", G_CALLBACK(close_fkb), self);*/
+    /*gtk_window_set_decorated (GTK_WINDOW (priv->window), FALSE);
+    gtk_window_set_default_size(GTK_WINDOW(priv->window),800,480);
+    g_signal_connect(priv->window, "destroy", G_CALLBACK(close_fkb), self);*/
   }
   hildon_im_ui_send_communication_message(priv->ui,
                                           HILDON_IM_CONTEXT_REQUEST_SURROUNDING_FULL);
-  hildon_im_ui_play_sound(priv->ui, HILDON_IM_FINGER_TRIGGER_SOUND);
+  /* Get current input mode. */
+  priv->input_mode = hildon_im_ui_get_current_input_mode( priv->ui );
   priv->saved_offset = 0;
 
   gtk_widget_show(GTK_WIDGET(priv->window));
@@ -551,7 +503,6 @@ hildon_im_fcitx_fkb_enable (HildonIMPlugin *plugin,gboolean init)
     PYInit();
     gtk_widget_grab_focus(priv->text_view);
   }
-  priv->return_close = FALSE; /* Browser and Xterminal use return_key as close*/
 }
 
 static void
@@ -564,7 +515,8 @@ hildon_im_fcitx_fkb_disable (HildonIMPlugin *plugin)
   self = HILDON_IM_FCITX_FKB(plugin);
   priv = HILDON_IM_FCITX_FKB_GET_PRIVATE(self);
 
-  gtk_widget_destroy(priv->window);
+  if(priv->window)
+    gtk_widget_destroy(priv->window);
 }
 /*static void
 hildon_im_fcitx_fkb_key_event(HildonIMPlugin *plugin,
@@ -676,7 +628,7 @@ static void show_candwords(HildonIMFcitxFKBPrivate *priv)
     gtk_button_set_label(GTK_BUTTON(priv->buttons[1][9]),(const gchar *)wd);
     gtk_widget_show(GTK_WIDGET(priv->buttons[3][2]));
   }else{
-    gtk_button_set_label(GTK_BUTTON(priv->buttons[1][9]),"");
+    gtk_button_set_label(GTK_BUTTON(priv->buttons[1][9])," ");
     gtk_widget_hide(GTK_WIDGET(priv->buttons[3][2]));
     return;
   }
@@ -734,20 +686,16 @@ text_button_clicked (GtkButton *button, gpointer user_data)
       strStringGet[0] = '\0';
       if(button == GTK_BUTTON(priv->buttons[3][5])){
         DoPYInput('1');
-        text = strStringGet;
       }else if(button == GTK_BUTTON(priv->buttons[3][6])){
         DoPYInput('2');
-        text = strStringGet;
       }else if(button == GTK_BUTTON(priv->buttons[3][4])){
         DoPYInput('3');
-        text = strStringGet;
       }else if(button == GTK_BUTTON(priv->buttons[3][7])){
         DoPYInput('4');
-        text = strStringGet;
       }else if(button == GTK_BUTTON(priv->buttons[3][3])){
         DoPYInput('5');
-        text = strStringGet;
       }
+      text = strStringGet;
       if(*text == '\0'){
         show_candwords(priv);
         return;
@@ -792,7 +740,7 @@ delete_button_clicked (gpointer user_data)
   priv = HILDON_IM_FCITX_FKB_GET_PRIVATE(self);
 
   priv->deleted = priv->deleted + 1;
-  if(priv->deleted > 1 && priv->deleted <10)
+  if(priv->deleted > 1 && priv->deleted <7)
     return TRUE;
   if(iPYInsertPoint){
     if(IRV_CLEAN  == DoPYInput(8))
@@ -825,7 +773,7 @@ static void
 button_update ( HildonIMFcitxFKBPrivate *priv, const gchar** keyboard_layout[])
 {
   GtkWidget *labelchild;
-  PangoFontDescription *font = pango_font_description_from_string("Nokia Sans 25");
+  PangoFontDescription *font = pango_font_description_from_string("Nokia Sans 24");
   gint i,j;
   for (i = 0; keyboard_layout[i] != NULL; i++)
   {
@@ -837,13 +785,13 @@ button_update ( HildonIMFcitxFKBPrivate *priv, const gchar** keyboard_layout[])
     }
   }
   pango_font_description_free(font);
-  const gchar *t = gtk_button_get_label(GTK_BUTTON(priv->buttons[3][0]));
-  if(*t == ',')
+  const gchar *t = gtk_button_get_label(GTK_BUTTON(priv->buttons[0][0]));
+  if(*t == 'q')
     gtk_button_set_label(GTK_BUTTON(priv->space_button)," ");
 }
 
 static void
-switch_button_toggled (GtkButton *button,
+switch_button_clicked (GtkButton *button,
                                gpointer user_data)
 {
   if(iPYInsertPoint)
@@ -928,7 +876,8 @@ return_button_clicked (GtkButton *button, gpointer user_data)
   self = HILDON_IM_FCITX_FKB(user_data);
   priv = HILDON_IM_FCITX_FKB_GET_PRIVATE(self);
 
-  delete_selection (self);
+  if(iPYInsertPoint || (priv->input_mode & HILDON_GTK_INPUT_MODE_MULTILINE))
+    delete_selection (self);
   buffer = gtk_text_view_get_buffer(GTK_TEXT_VIEW(priv->text_view));
   if(chinese_mode && iPYInsertPoint){
     if (hildon_im_ui_get_commit_mode(priv->ui) == HILDON_IM_COMMIT_REDIRECT){
@@ -944,10 +893,8 @@ return_button_clicked (GtkButton *button, gpointer user_data)
     show_candwords(priv);
     return;
   }
-  Window win = hildon_im_ui_get_input_window( priv->ui );
 
-  if(get_x11win_prop(win)){
-    priv->return_close = TRUE;
+  if((priv->input_mode & HILDON_GTK_INPUT_MODE_MULTILINE) == 0){
     close_fkb(priv->window,self);
     return;
   }
@@ -983,7 +930,7 @@ static void backspace_pressed(GtkButton *button,gpointer user_data)
   priv = HILDON_IM_FCITX_FKB_GET_PRIVATE(self);
   priv->deleted = 0;
   if (delete_button_clicked(self))
-    priv->timeout_handler = g_timeout_add(75,delete_button_clicked,self);
+    priv->timeout_handler = g_timeout_add(100,delete_button_clicked,self);
 }
 static void
 close_fkb (GtkWidget *button, gpointer user_data)
@@ -992,7 +939,6 @@ close_fkb (GtkWidget *button, gpointer user_data)
   HildonIMFcitxFKBPrivate *priv;
   GtkTextIter start,end;
   GtkTextBuffer *buffer;
-  gboolean newline = FALSE;
 
   g_return_if_fail (HILDON_IS_IM_FCITX_FKB (user_data));
   self = HILDON_IM_FCITX_FKB(user_data);
@@ -1002,42 +948,18 @@ close_fkb (GtkWidget *button, gpointer user_data)
     ResetPYStatus();
     show_candwords(priv);
   }
-  if (GTK_WIDGET_VISIBLE(GTK_WIDGET(priv->window)))
-  {
-    gtk_widget_hide(GTK_WIDGET(priv->window));
-  }
+  hildon_im_ui_restore_previous_mode(priv->ui);
+
   buffer = gtk_text_view_get_buffer(GTK_TEXT_VIEW(priv->text_view));
   gtk_text_buffer_get_iter_at_mark(buffer, &start,
                                    gtk_text_buffer_get_selection_bound(buffer));
-  gint offset = gtk_text_iter_get_offset(&start);
   gtk_text_buffer_get_bounds(buffer, &start, &end);
   gchar *text = gtk_text_buffer_get_text (buffer, &start, &end, FALSE);
-  GString *newstr = g_string_new(text);
-  if(offset == gtk_text_iter_get_offset(&end)){
-    gsize len = newstr->len;
-    if(len > 0){
-      if(newstr->str[len-1]=='\n'){
-        newstr = g_string_truncate(newstr, len-1);
-        newline = TRUE;
-      }
-    }
-  }
-  if (hildon_im_ui_get_commit_mode(priv->ui) == HILDON_IM_COMMIT_SURROUNDING)
-  {
-    hildon_im_ui_send_surrounding_content( priv->ui, newstr->str);
-  }else if(newline){
+  if(hildon_im_ui_get_commit_mode(priv->ui) == HILDON_IM_COMMIT_SURROUNDING)
+    hildon_im_ui_send_surrounding_content( priv->ui, text);
+  if(button == priv->window){
     hildon_im_ui_send_communication_message (priv->ui,
-                                            HILDON_IM_CONTEXT_HANDLE_BACKSPACE);
-  }
-  g_string_free(newstr,FALSE);
-  hildon_im_ui_restore_previous_mode(priv->ui);
-  if(newline || priv->return_close){
-    if (hildon_im_ui_get_commit_mode(priv->ui) == HILDON_IM_COMMIT_SURROUNDING)
-      hildon_im_ui_send_communication_message (priv->ui,
-                                            HILDON_IM_CONTEXT_HANDLE_ENTER);
-    else hildon_im_ui_send_communication_message (priv->ui,
                                             HILDON_IM_CONTEXT_ENTER_ON_FOCUS);
-    priv->return_close = FALSE;
   }
 }
 static gboolean textview_copy (GtkButton *button, gpointer data)
@@ -1119,40 +1041,40 @@ populate_window (HildonIMFcitxFKB *self)
 {
   HildonIMFcitxFKBPrivate *priv;
   GtkWidget *parea, *labelchild;
-  PangoFontDescription *font = pango_font_description_from_string("Nokia Sans 25");
+  gint button_width = gdk_screen_width() / 10;
+  PangoFontDescription *font = pango_font_description_from_string("Nokia Sans 24");
 
   priv = HILDON_IM_FCITX_FKB_GET_PRIVATE(self);
   priv->vbox = gtk_vbox_new(FALSE, 1);
 
   priv->buttons[1][9] = gtk_button_new();
   gtk_button_set_focus_on_click(GTK_BUTTON(priv->buttons[1][9]),FALSE);
-  gtk_widget_set_size_request(priv->buttons[1][9], BUTTON_WIDTH*100, BUTTON_HIGHT);
+  gtk_widget_set_size_request(priv->buttons[1][9], button_width*10, BUTTON_HEIGHT);
   g_signal_connect (priv->buttons[1][9], "clicked", G_CALLBACK(close_fkb), self);
   gtk_box_pack_start(GTK_BOX(priv->vbox), priv->buttons[1][9], FALSE, FALSE, 0);
 
   priv->text_view = gtk_text_view_new();
   gtk_text_view_set_editable (GTK_TEXT_VIEW (priv->text_view), TRUE);
-  gtk_text_view_set_wrap_mode(GTK_TEXT_VIEW(priv->text_view), GTK_WRAP_WORD_CHAR);
+  gtk_text_view_set_wrap_mode(GTK_TEXT_VIEW(priv->text_view), GTK_WRAP_CHAR);
   gtk_text_view_set_cursor_visible(GTK_TEXT_VIEW(priv->text_view), TRUE);
   gtk_text_view_set_left_margin(GTK_TEXT_VIEW(priv->text_view), 10);
-  gtk_text_view_set_right_margin(GTK_TEXT_VIEW(priv->text_view), 10);
 
   gtk_widget_modify_font(GTK_WIDGET(priv->text_view),font);
   g_signal_connect(G_OBJECT(priv->text_view), "button-release-event",
                          G_CALLBACK(textview_button_release_cb), self);
 
   GtkWidget *menu = gtk_menu_new();
-  GtkWidget *copy_menu = gtk_menu_item_new_with_label("复制");
-  GtkWidget *cut_menu = gtk_menu_item_new_with_label("剪切");
-  GtkWidget *paste_menu = gtk_menu_item_new_with_label("粘帖");
-  gtk_menu_shell_append(GTK_MENU_SHELL(menu), copy_menu);
-  gtk_menu_shell_append(GTK_MENU_SHELL(menu), cut_menu);
-  gtk_menu_shell_append(GTK_MENU_SHELL(menu), paste_menu);
-  g_signal_connect(G_OBJECT(copy_menu), "activate",
+  GtkWidget *copy_item = gtk_menu_item_new_with_label("复制");
+  GtkWidget *cut_item = gtk_menu_item_new_with_label("剪切");
+  GtkWidget *paste_item = gtk_menu_item_new_with_label("粘贴");
+  gtk_menu_shell_append(GTK_MENU_SHELL(menu), copy_item);
+  gtk_menu_shell_append(GTK_MENU_SHELL(menu), cut_item);
+  gtk_menu_shell_append(GTK_MENU_SHELL(menu), paste_item);
+  g_signal_connect(G_OBJECT(copy_item), "activate",
                          G_CALLBACK(textview_copy), self);
-  g_signal_connect(G_OBJECT(cut_menu), "activate",
+  g_signal_connect(G_OBJECT(cut_item), "activate",
                          G_CALLBACK(textview_cut), self);
-  g_signal_connect(G_OBJECT(paste_menu), "activate",
+  g_signal_connect(G_OBJECT(paste_item), "activate",
                          G_CALLBACK(textview_paste), self);
   gtk_widget_tap_and_hold_setup(priv->text_view, menu, NULL, 0);
   gtk_widget_show_all (menu);
@@ -1161,7 +1083,6 @@ populate_window (HildonIMFcitxFKB *self)
   gtk_scrolled_window_set_policy (GTK_SCROLLED_WINDOW (parea),
                                   GTK_POLICY_NEVER,
                                   GTK_POLICY_AUTOMATIC);
-  //parea = hildon_pannable_area_new();
   gtk_container_add (GTK_CONTAINER (parea), priv->text_view);
   gtk_container_add (GTK_CONTAINER (priv->vbox), parea);
 
@@ -1182,7 +1103,7 @@ populate_window (HildonIMFcitxFKB *self)
     GtkWidget *hbox  = gtk_hbox_new(FALSE, 0);
     if (i == 1){
         GtkWidget *label = gtk_label_new(" ");
-        gtk_widget_set_size_request(label, BUTTON_WIDTH*0.5, BUTTON_HIGHT);
+        gtk_widget_set_size_request(label, button_width*0.5, BUTTON_HEIGHT);
         gtk_box_pack_start(GTK_BOX(hbox), label, FALSE, FALSE, 0);
     }
     else if (i == 2){
@@ -1191,7 +1112,7 @@ populate_window (HildonIMFcitxFKB *self)
         gtk_container_add (GTK_CONTAINER (priv->shift_button),
                           gtk_image_new_from_icon_name("keyboard_shift",
                                                        -1));
-        gtk_widget_set_size_request(priv->shift_button, BUTTON_WIDTH*1.5, BUTTON_HIGHT);
+        gtk_widget_set_size_request(priv->shift_button, button_width*1.5, BUTTON_HEIGHT);
         gtk_box_pack_start(GTK_BOX(hbox), priv->shift_button, FALSE, FALSE, 0);
         g_signal_connect(G_OBJECT(priv->shift_button), "toggled", G_CALLBACK(shift_symbols_buttons_toggled), self);
     }
@@ -1200,7 +1121,7 @@ populate_window (HildonIMFcitxFKB *self)
 
       priv->symbols_button = gtk_toggle_button_new_with_label ("符");
       gtk_button_set_focus_on_click(GTK_BUTTON(priv->symbols_button),FALSE);
-      gtk_widget_set_size_request(priv->symbols_button, BUTTON_WIDTH*1.5, BUTTON_HIGHT);
+      gtk_widget_set_size_request(priv->symbols_button, button_width*1.5, BUTTON_HEIGHT);
       gtk_box_pack_start(GTK_BOX(hbox), priv->symbols_button, FALSE, FALSE, 0);
       g_signal_connect(priv->symbols_button, "toggled",
                    G_CALLBACK(shift_symbols_buttons_toggled), self);
@@ -1209,9 +1130,9 @@ populate_window (HildonIMFcitxFKB *self)
 
       priv->mode_button = gtk_button_new_with_label (chinese_mode?"拼":"英");
       gtk_button_set_focus_on_click(GTK_BUTTON(priv->mode_button),FALSE);
-      gtk_widget_set_size_request(priv->mode_button, BUTTON_WIDTH, BUTTON_HIGHT);
+      gtk_widget_set_size_request(priv->mode_button, button_width, BUTTON_HEIGHT);
       gtk_box_pack_start(GTK_BOX(hbox), priv->mode_button, FALSE, FALSE, 0);
-      g_signal_connect(priv->mode_button, "clicked", G_CALLBACK(switch_button_toggled), self);
+      g_signal_connect(priv->mode_button, "clicked", G_CALLBACK(switch_button_clicked), self);
 
       /* Space button */
 
@@ -1226,7 +1147,7 @@ populate_window (HildonIMFcitxFKB *self)
         gtk_button_set_focus_on_click(GTK_BUTTON(priv->buttons[i][j]),FALSE);
         labelchild = gtk_bin_get_child(GTK_BIN(priv->buttons[i][j]));
         gtk_widget_modify_font(GTK_WIDGET(labelchild),font);
-        gtk_widget_set_size_request(GTK_WIDGET(priv->buttons[i][j]),BUTTON_WIDTH,BUTTON_HIGHT);
+        gtk_widget_set_size_request(GTK_WIDGET(priv->buttons[i][j]),button_width,BUTTON_HEIGHT);
         g_signal_connect(G_OBJECT(priv->buttons[i][j]), "pressed", G_CALLBACK(text_button_clicked), self);
         gtk_box_pack_start(GTK_BOX(hbox), GTK_WIDGET(priv->buttons[i][j]), FALSE, FALSE, 0);
     }
@@ -1236,7 +1157,7 @@ populate_window (HildonIMFcitxFKB *self)
         gtk_button_set_image(GTK_BUTTON(priv->delete_button),
                           gtk_image_new_from_icon_name("keyboard_backspace",
                                                        1));
-        gtk_widget_set_size_request(priv->delete_button, BUTTON_WIDTH*1.5, BUTTON_HIGHT);
+        gtk_widget_set_size_request(priv->delete_button, button_width*1.5, BUTTON_HEIGHT);
         g_signal_connect(G_OBJECT(priv->delete_button), "pressed", G_CALLBACK(backspace_pressed), self);
         g_signal_connect(G_OBJECT(priv->delete_button), "released", G_CALLBACK(backspace_released), self);
         gtk_box_pack_start(GTK_BOX(hbox), priv->delete_button, FALSE, FALSE, 0);
@@ -1249,7 +1170,7 @@ populate_window (HildonIMFcitxFKB *self)
       gtk_button_set_image(GTK_BUTTON(priv->return_button),
                           gtk_image_new_from_icon_name("keyboard_enter",
                                                        -1));
-      gtk_widget_set_size_request(priv->return_button, BUTTON_WIDTH*1.5, BUTTON_HIGHT);
+      gtk_widget_set_size_request(priv->return_button, button_width*1.5, BUTTON_HEIGHT);
       gtk_box_pack_start(GTK_BOX(hbox), priv->return_button, FALSE, FALSE, 0);
       g_signal_connect(priv->return_button, "clicked", G_CALLBACK(return_button_clicked), self);
     }
@@ -1258,8 +1179,8 @@ populate_window (HildonIMFcitxFKB *self)
   gtk_container_add (GTK_CONTAINER(priv->window), priv->vbox);
   pango_font_description_free(font);
   font = pango_font_description_from_string("Nokia Sans 36");
-  gtk_widget_modify_font(GTK_WIDGET(GTK_BIN(copy_menu)->child),font);
-  gtk_widget_modify_font(GTK_WIDGET(GTK_BIN(cut_menu)->child),font);
-  gtk_widget_modify_font(GTK_WIDGET(GTK_BIN(paste_menu)->child),font);
+  gtk_widget_modify_font(GTK_WIDGET(GTK_BIN(copy_item)->child),font);
+  gtk_widget_modify_font(GTK_WIDGET(GTK_BIN(cut_item)->child),font);
+  gtk_widget_modify_font(GTK_WIDGET(GTK_BIN(paste_item)->child),font);
   pango_font_description_free(font);
 }
